import type { PromptAnswers, PromptQuestion, Logger, PromptQuestions, Task, QueuedAdapter } from '@yeoman/types';
import { type PromptModule } from 'inquirer';
export type DummyPromptCallback = (answer: any, { question, answers }: {
    question: PromptQuestion;
    answers: PromptAnswers;
}) => any;
export type DummyPromptOptions = {
    mockedAnswers?: PromptAnswers;
    callback?: DummyPromptCallback;
    throwOnMissingAnswer?: boolean;
};
type SpyFactory<SpyType = any> = ({ returns }: {
    returns?: any;
}) => SpyType;
export type TestAdapterOptions<SpyType = any> = DummyPromptOptions & {
    log?: any;
    spyFactory?: SpyFactory<SpyType>;
};
export type DefineTestAdapterConfig = Pick<TestAdapterOptions, 'log' | 'spyFactory' | 'throwOnMissingAnswer'>;
export declare class DummyPrompt {
    answers: PromptAnswers;
    question: PromptQuestion;
    callback: DummyPromptCallback;
    throwOnMissingAnswer: boolean;
    constructor(question: PromptQuestion, _rl: any, answers: PromptAnswers, options?: DummyPromptOptions);
    run(): Promise<any>;
}
export declare const defineConfig: (config: DefineTestAdapterConfig) => DefineTestAdapterConfig;
export declare const getConfig: () => {
    log?: any;
    spyFactory?: SpyFactory<any> | undefined;
    throwOnMissingAnswer?: boolean | undefined;
};
/**
 * @experimental
 */
export declare class TestAdapter<LogType extends Logger = Logger, SpyType = any> implements QueuedAdapter {
    promptModule: PromptModule;
    diff: any & SpyType;
    log: LogType & SpyType;
    registerDummyPrompt: (promptName: string, customPromptOptions?: DummyPromptOptions) => PromptModule;
    private readonly spyFactory;
    constructor(options?: TestAdapterOptions<SpyType>);
    queue<TaskResultType>(fn: Task<TaskResultType>): Promise<void | TaskResultType>;
    progress<ReturnType>(fn: (progress: {
        step: (prefix: string, message: string, ...args: any[]) => void;
    }) => ReturnType, _options?: {
        disabled?: boolean | undefined;
        name?: string | undefined;
    } | undefined): Promise<void | ReturnType>;
    close(): void;
    prompt<A extends PromptAnswers = PromptAnswers>(questions: PromptQuestions<A>, initialAnswers?: Partial<A> | undefined): Promise<A>;
}
export {};
